import SVG from "svg.js"
import $ from "jquery"
import SvgDrag from "svg.draggable.js"
import svgIntersections from "svg-intersections"
var intersect = svgIntersections.intersect;
var shape = svgIntersections.shape;

SVG.on(window, 'resize', function() { svg.spof() });
var svg = SVG('drawing');

var typesCards = ["concept", "instance"];
var typesRelations = ["is a", "part of", "instance of"];

/**
 *
 * @param area
 * @constructor
 */
var Map = function (area) {
    this.area = area;
    this.relations = [];
    this.cards = [];
    this.connectTo = function (card_1, card_2, relationType) {
        
    };
    this.addCard = function (card) {
        this.cards.push(card);
    };
    this.addRelation = function (relation) {
        this.relations.push(relation);
    };
    this.designAll = function () {
        for(var i in this.cards) {
            this.cards[i].design();
        }
        for(var i in this.relations) {
            this.relations[i].design();
        }
    };
    this.isCardRegistered = function (card) {
        for(var i in this.cards) {
            if(this.cards[i] == card)
                return true;
        }
        return false;
    };
    this.isRelationRegistered = function (relation) {
        for(var i in this.relations) {
            if(this.relations[i] == relation)
                return true;
        }
        return false;
    }
};

/**
 *
 * @param map
 * @param name
 * @param type
 * @constructor
 */
var Card = function (map, name, type) {
    this.svgRect = null;
    this.name = name;
    this.type = type;
    this.map = map;
    this.design = function () {
        if(this.svgRect == null) {
            this.svgRect = this.map.area.rect(100, 65).fill("#e96c00").draggable();
        }
    };
    this.register = function () {
        if(!this.map.isCardRegistered(this))
            this.map.addCard(this);
        else
            console.log("Carte déjà enregistrée");
    };
    this.register();
};

/**
 *
 * @param map
 * @param from
 * @param to
 * @param type
 * @constructor
 */
var Relation = function (map, from, to, type) {
    this.map = map;
    this.from = from;
    this.to = to;
    this.type = type;
    this.svgLineStart = null;
    this.svgLineEnd = null;
    this.svgLabel = null;
    this.design = function () {
        if(this.svgLabel == null && this.svgLineEnd == null && this.svgLineStart == null){
            this.svgLabel = this.map.area.text(this.type).font({
                size: 20
            }).draggable();
            this.svgLineStart = this.map.area.line(this.from.svgRect.cx(), this.from.svgRect.cy(), this.svgLabel.cx(), this.svgLabel.cy()).stroke({ width: 1 });
            this.svgLineEnd = this.map.area.line(this.svgLabel.cx(), this.svgLabel.cy(), this.to.svgRect.cx(), this.to.svgRect.cy()).stroke({ width: 1 });
            this.from.svgRect.on("dragmove", this.connect, this);
            this.to.svgRect.on("dragmove", this.connect, this);
            this.svgLabel.on("dragmove", this.connect, this);
        }
    };
    this.connect = function (relation) {
        this.svgLineStart.plot(
            this.from.svgRect.cx(),
            this.from.svgRect.cy(),
            this.svgLabel.x(),
            this.svgLabel.y()
        );
        this.svgLineEnd.plot(
            this.svgLabel.x(),
            this.svgLabel.y(),
            this.to.svgRect.cx(),
            this.to.svgRect.cy()
        );

        var startIntersect = intersect(
            shape("line", {x1: this.svgLineStart.attr("x1"), y1:this.svgLineStart.attr("y1"), x2:this.svgLineStart.attr("x2"), y2:this.svgLineStart.attr("y2")}),
            shape("rect", {x:this.from.svgRect.x(), y:this.from.svgRect.y(), width:100, height:60})
        );

        var endIntersect = intersect(
            shape("line", {x1: this.svgLineEnd.attr("x1"), y1:this.svgLineEnd.attr("y1"), x2:this.svgLineEnd.attr("x2"), y2:this.svgLineEnd.attr("y2")}),
            shape("rect", {x:this.to.svgRect.x(), y:this.to.svgRect.y(), width:100, height:60})
        );

        if(startIntersect.points.length > 0) {
            this.svgLineStart.plot(
                startIntersect.points[0].x,
                startIntersect.points[0].y,
                this.svgLabel.x(),
                this.svgLabel.y()
            );
        }
        if(endIntersect.points.length > 0) {
            this.svgLineEnd.plot(
                this.svgLabel.x(),
                this.svgLabel.y(),
                endIntersect.points[0].x,
                endIntersect.points[0].y
            );
        }
    };
    this.register = function () {
        if(!this.map.isRelationRegistered(this))
            this.map.addCard(this);
        else
            console.log("Relation déjà enregistrée");
    };
    this.register();
};

/****************************************************/

var map = new Map(svg);

var c1 = new Card(map, "Carte de test", typesCards[0]);
var c2 = new Card(map, "Autre carte de test", typesCards[1]);
var c3 = new Card(map, "Dernière carte de test", typesCards[1]);
var r1 = new Relation(map, c1, c2, typesRelations[0]);
var r2 = new Relation(map, c2, c3, typesRelations[1]);
map.designAll();



